function Splines_Interpolantes(t,y,CI,S1a,S2a,CD,S1b,S2b,opx,x,opg1,opg2,opg3,opg4)


% Esta funcin busca hallar las ecuaciones que definen el spline y dibujar
% las grficas de la curva, su primera y segunda derivada, asi como los
% valores obtenidos al interpolar y su grfica. Creando ficheros que
% recojan la informacin.
%
% Splines_Interpolantes(t,y,CI,S1a,S2a,CD,S1b,S2b,opx,x,opg1,opg2,opg3,opg4)
%
% Variables de entrada:
% t:abscisas de la tabla de valores de los nodos
% y:ordenadas de la tabla de valores de los nodos
% CI:condicin de contorno en el extremo izquierdo
% CD:condicin de contorno en  el extremo derecho
%
% Posibles condiciones de contorno:
% 1- De primer tipo: valores de la primera derivada
% 2- De segundo tipo: valores de la segunda derivada
% 3- De tercer tipo: peridicas de periodo T=b-a
%    S'(a)=S'(b); S''(a)=S''(b). Si CI=3->CD=3
% 4- De cuarto tipo: Tres veces diferenciable en los puntos t2 y/o tm-1
%
% S1a:valor de la primera derivada en el extremo izquierdo
% S2a:valor de la segunda derivada en el extremo izquierdo
% S1b:valor de la primera derivada en el extremo derecho
% S2b:valor de la segunda derivada en el extremo derecho
%
% opx:parmetro que nos indica si debemos calcular o no los valores interpolados
% 's':se calculan; 'n': no se calculan
% x:retculo de puntos donde queremos evaluar la funcin polinmica a trozos
% si opgI=1 entonces dibujar la grfica
% opg1:parmetro que nos indica se debemos dibujar o no la grfica de los valoes interpolados
% opg2:parmetro que nos indica se debemos dibujar o no la grfica de los splines
% opg3:parmetro que nos indica se debemos dibujar o no la grfica de las primeras derivadas
% opg4:parmetro que nos indica se debemos dibujar o no la grfica de las segundas derivadas
%
%EJEMPLO:
%t=[1,3,4,6]
%y=[2,-1,3,1]
%CI=2 ; CD=1
%S1a=0; S2b=0 (no se van a utilizar, da igual el dato)
%S2a=2; S1b=2
%opx='s'
%x=[0:0.2:6]
%
%Splines_Interpolantes([1,3,4,6],[2,-1,3,1],2,0,2,1,2,2,'s',[0:0.2:6],1,1,1,1)




% Nmero de puntos en la tabla
m=length(t);

% Comprobamos que las entradas son correctas

if m<3
    uiwait(msgbox('El nmero de puntos de control debe ser mayor que 2', 'Mensaje de error',...
        'error','modal'));
    return;
elseif CI==0 | CD==0
    uiwait(msgbox('Falta especificar alguna de las condiciones de frontera', 'Mensaje de error',...
        'error','modal'));
    return;
elseif CI==3 & CD~=3
    uiwait(msgbox('Si CI=3 entonces CD debe ser tambin 3', 'Mensaje de error',...
        'error','modal'));
    return;
elseif CD==3 & CI~=3
    uiwait(msgbox('Si CD=3 entonces CI debe ser tambin 3', 'Mensaje de error',...
        'error','modal'));
    return;
elseif CD==3 & y(1)~=y(m)
    uiwait(msgbox('Para que la funcin sea peridica debe valer lo mismo en los extremos', 'Mensaje de error',...
        'error','modal'));
    return;
elseif m==3 & CI==4 & CD==4
    uiwait(msgbox('Con 3 nodos la condicin CI=4 ya implica CD=4 y nos falta una condicin', 'Mensaje de error',...
        'error','modal'));
    return;
end


% Definimos el vector h de distancias entre las abscisas
h=diff(t,1);

% Inicializamos
z=zeros(1,m);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Clculo de las derivadas segundas
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Definimos la matriz de coeficientes del sistema lineal
A=zeros(m,m);
% Definimos el trmino independiente
B=zeros(m,1);



% Definimos la primera ecuacin del sistema dependiendo de CI

if CI==1
    A(1,1)=-h(1)/3;
    A(1,2)=-h(1)/6;
    B(1)=S1a+(y(1)-y(2))/h(1);
elseif CI==2
    B(1)=S2a;
    A(1,1)=1;
elseif CI==3
    % primera ecuacin
    A(1,1)=h(1)/3; A(1,2)=h(1)/6; A(1,m-1)=h(m-1)/6; A(1,m)=h(m-1)/3;
    B(1)=(y(m-1)-y(m))/h(m-1)+(y(2)-y(1))/h(1);
    % ltima ecuacin
    A(m,1)=1; A(m,m)=-1;
elseif CI==4
    A(1,1)=-h(2); A(1,2)=h(1)+h(2); A(1,3)=-h(1);
end


% Definimos las m-2 ecuaciones intermedias


for i=2:m-1
    A(i,i-1)=h(i-1);
    A(i,i)=2*(h(i-1)+h(i));
    A(i,i+1)=h(i);
end

% A continuacin definimos la matriz B

for i=2:m-1
    B(i)=6/h(i)*(y(i+1)-y(i))-6/h(i-1)*(y(i)-y(i-1));
end



% Definimos la ltima ecuacin del sistema dependiendo de CD

if CD==1
    A(m,m-1)=h(m-1)/6; A(m,m)=h(m-1)/3;
    B(m)=S1b+(y(m-1)-y(m))/h(m-1);
elseif CD==2
    B(m)=S2b;
    A(m,m)=1;
elseif CD==4
    A(m,m-2)=-h(m-1); A(m,m-1)=h(m-1)+h(m-2); A(m,m)=-h(m-2);
end


% Resolucin del sistema


if abs(det(A))<=10^(-14)
    uiwait(msgbox('El problema puede no tener solucin, la matriz est cerca de no ser invertible',...
        'Mensaje de error','error','modal'));
    return;
else
    z(1:m)=A\B;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Evaluacin de los splines en x
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if opx=='s'

    % Encuentra el trozo polinmico a evaluar segn cada entrada de x
    if x(1)==t(1)
        minim=y(1); maxim=y(1);
        Si(1)=y(1);
    else
        ind=find((x(1)>t)==0);
        ind=ind(1)-1;
        ind=ind(1);
        % Aplica la frmula para calcular S_ind(x)
        Si(1)=z(ind+1)/(6*h(ind))*(x(1)-t(ind))^3+...
            z(ind)/(6*h(ind))*(t(ind+1)-x(1))^3+(y(ind+1)/h(ind)-...
            z(ind+1)*h(ind)/6)*(x(1)-t(ind))+(y(ind)/h(ind)-...
            z(ind)*h(ind)/6)*(t(ind+1)-x(1));
        minim=Si; maxim=Si;
    end

    for i=2:length(x)
        ind=find((x(i)>t)==0);
        ind=ind(1)-1;
        ind=ind(1);
        % Aplica la frmula para calcular S_ind(x)
        Si(i)=z(ind+1)/(6*h(ind))*(x(i)-t(ind))^3+...
            z(ind)/(6*h(ind))*(t(ind+1)-x(i))^3+(y(ind+1)/h(ind)-...
            z(ind+1)*h(ind)/6)*(x(i)-t(ind))+(y(ind)/h(ind)-...
            z(ind)*h(ind)/6)*(t(ind+1)-x(i));
        if Si(i)<minim
            minim=Si(i);
        elseif Si(i)>maxim
            maxim=Si(i);
        end
    end

    if opg1==1
        figure(1);
        %Dibuja los puntos de control en la grfica
        b1=plot(t,y,'ko','MarkerSize',2,'LineWidth',3);
        hold on;
        %Dibuja los valores interpolados en la grfica
        b2=plot(x,Si,'ro','MarkerSize',3);
        hold on;
        axis([t(1)-0.1*(t(2)-t(1)) t(length(t))+...
            0.1*(t(2)-t(1)) min(min(y),minim)-...
            0.1*(max(y)-min(y)) max(max(y),maxim)+0.1*(max(y)-min(y))]);
        axis equal;
        legend([b1,b2],'Puntos de Control','Valores Interpolados');
    end
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Polinomios S_i(x)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

syms r;



for i=1:m-1
    S(i)=z(i+1)/(6*h(i))*(r-t(i))^3+z(i)/(6*h(i))*(t(i+1)-r)^3+...
        (y(i+1)/h(i)-z(i+1)*h(i)/6)*(r-t(i))+(y(i)/h(i)-...
        z(i)*h(i)/6)*(t(i+1)-r);
end


if opg2==1
    %Dibuja los puntos de control en la grfica
    figure(2);
    b1=plot(t,y,'ko','MarkerSize',2,'LineWidth',3);
    hold on;

    %Dibuja cada uno de los polinomios
    Ms=0;
    ms=0;
    for i=1:m-1
        b3=ezplot(S(i),[t(i),t(i+1)]);
        auxX=t(i):(t(i+1)-t(i))/10:t(i+1);
        auxY=subs(S(i),auxX); MauxY=max(auxY); mauxY=min(auxY);
        if MauxY>Ms
            Ms=MauxY;
        end
        if mauxY<ms
            ms=mauxY;
        end
    end
    axis([t(1)-0.1*(t(2)-t(1)) t(length(t))+...
        0.1*(t(2)-t(1))  ms-0.1*(Ms-ms)  Ms+0.1*(Ms-ms)]);
    axis equal;
    legend([b1,b3],'Puntos de Control','Splines Cbicos');
    title('Ajuste por Splines Cbicos Interpolantes');
end


if opg3==1
    %Dibuja cada una de las primeras derivadas
    figure(3);
    hold on
    Ms=0;
    ms=0;
    S1=diff(S,1);
    for i=1:m-1
        b4=ezplot(S1(i),[t(i),t(i+1)]);
        auxX=t(i):(t(i+1)-t(i))/10:t(i+1);
        auxY=subs(S1(i),auxX); MauxY=max(auxY); mauxY=min(auxY);
        if MauxY>Ms
            Ms=MauxY;
        end
        if mauxY<ms
            ms=mauxY;
        end
    end
    axis([t(1)-0.1*(t(2)-t(1)) t(length(t))+...
        0.1*(t(2)-t(1))  ms-0.1*(Ms-ms)  Ms+0.1*(Ms-ms)]);
    legend(b4,'Primera Derivada Splines Cbicos');
    title('Primera Derivada de los Splines Cbicos Interpolantes');
end


if opg4==1
    %Dibuja cada una de las segundas derivadas
    figure(4)
    hold on
    S2=diff(S,2);
    Msd=subs(S2(1),t(1));
    msd=subs(S2(1),t(1));
    for i=1:m-1
        b5=ezplot(S2(i),[t(i),t(i+1)]);
        aux=subs(S2(i),t(i+1));
        if aux>Msd
            Msd=aux;
        elseif aux<msd
            msd=aux;
        end
    end
    axis([t(1)-0.1*(t(2)-t(1)) t(length(t))+...
        0.1*(t(2)-t(1)) msd-0.1*(Msd-msd)  Msd+0.1*(Msd-msd)]);
    legend(b5,'Segunda Derivada Splines Cbicos');
    title('Segunda Derivada de los Splines Cbicos Interpolantes');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  SALIDA DE RESULTADOS A UN FICHERO
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%  Se abre o crea un archivo y se escribe en l para los resultados de valores interpolados 

if opx=='s'
    fid=fopen('resul_valores_interpolados.txt','w');
    fprintf(fid,'**************************************************************************\n');
    hora=clock;
    fprintf(fid,'Resultado ejecutado el dia %d-%d-%d a las %d : %d : %d\n',...
        hora(3),hora(2),hora(1),hora(4),hora(5),fix(hora(6)));
    fprintf(fid,'**************************************************************************\n\n');
    for i=1:length(x)
        fprintf(fid,'%f %f \n',x(i),Si(i));
    end
    fclose(fid);
end

%  Se abre o crea un archivo y se escribe en l para los resultados de los polinomios del Spline 

fid=fopen('polinomios_splines.txt','w');
fprintf(fid,'**************************************************************************\n');
hora=clock;
fprintf(fid,'Resultado ejecutado el dia %d-%d-%d a las %d : %d : %d\n',...
    hora(3),hora(2),hora(1),hora(4),hora(5),fix(hora(6)));
fprintf(fid,'**************************************************************************\n\n');
signos='----';
for i=1:length(t)-1
    coefi=sym2poly(expand(S(i)));
    for j=1:4
        if coefi(j)>=0
            signos(j)='+';
        end
    end
    fprintf(fid,' %c %f x^3 %c %f x^2 %c %f x %c %f  \n',...
        signos(1),abs(coefi(1)),signos(2),abs(coefi(2)),signos(3),...
        abs(coefi(3)),signos(4),abs(coefi(4)));
end
fprintf(fid,'\n');
fclose(fid);

